<?php
// $Id: taxonomy_vtn.pages.inc,v 1.1.2.25 2008/12/22 00:41:27 alexiswilke Exp $

/**
 * @file
 * pages for display vocabularies, terms and nodes
 *
 */


/**
 * Get a list of vocabularies or terms to include/exclude.
 * In case users do not put spaces, we remove them here.
 *
 * @param[in] $varname1  The name of the global variable
 * @param[in] $varname2  The name of the local variable when available
 *
 * @return NULL or an array of identifiers
 */
function _taxonomy_vtn_get_list($varname1, $varname2 = NULL) {
  $result = NULL;

  $r = explode(',', str_replace(' ', '', variable_get($varname1, '')));

  if ($varname2) {
    $r = array_merge($r, explode(',', str_replace(' ', '', variable_get($varname2, ''))));
  }

  foreach($r as $id) {
    if (is_numeric($id)) {
      $result[$id] = $id;
    }
  }

  return $result;
}

/**
 * Implementation of taxonomy_vtn_show_vocabularies().
 */
function taxonomy_vtn_show_vocabularies() {

  // all vocabulary ids
  $all_vocs = taxonomy_get_vocabularies();

  // keep only those the user wants to include
  $include_vocs = _taxonomy_vtn_get_list('taxonomy_vtn_vocabularies_include');
  if ($include_vocs) {
    foreach($include_vocs as $include_vocs_id) {
      if (isset($all_vocs[$include_vocs_id])) {
        $vocs[$include_vocs_id] = $all_vocs[$include_vocs_id];
      }
    }
  }
  else {
    $vocs = $all_vocs;
  }

  // remove those the user never wants
  $omit_vocs = _taxonomy_vtn_get_list('taxonomy_vtn_vocabularies_omit');
  if ($omit_vocs) {
    foreach ($omit_vocs as $omit_vocs_id) {
      unset($vocs[$omit_vocs_id]);
    }
  }

  if ( (count($vocs) == 1) && (variable_get('taxonomy_vtn_vocabularies_goto_terms_if_one', 0) == 1) ) {
    drupal_goto('taxonomy_vtn/voc/'. key($vocs));
  }

  // Check if the page hasn't already been cached. If not, create and cache it.
  $cc = cache_get('taxonomy_vtn_show_vocs');
  $cache_data = $cc->data;
  if ($cache_data && variable_get('taxonomy_vtn_caching', 0) == 1) {
    $output = $cache_data;
    //print 'z cache';
    return $output;
  }

  $output = '<div class="taxonomy-vtn-vocabulary-page">';
  if ($vocs) {

    $show_empty_vocs = variable_get('taxonomy_vtn_vocabularies_show_empty_vocs', 1);
    $show_index = variable_get('taxonomy_vtn_vocabularies_show_voc_index', 0);
    $show_count_terms = variable_get('taxonomy_vtn_vocabularies_show_count_terms', 1);
    $show_voc_desc = variable_get('taxonomy_vtn_vocabularies_show_voc_desc', 1);
    $count_group_vocabularies =  variable_get('taxonomy_vtn_vocabularies_count_group_vocabularies', 1);
    $count_column_vocs =  variable_get('taxonomy_vtn_vocabularies_count_column_vocs', 2);
    $arr = array();
    $is_desc = false;

    foreach ( $vocs as $vid => $voc_obj ) {
      // TODO: this not exclude no published
      $terms = taxonomy_get_tree($vid);
      $tcount = count($terms);

      if ($show_count_terms) {
        $count_terms = ' ('. $tcount .')';
      }

      if ($show_voc_desc) {
        $voc_desc = '<br /><span class="description">'. check_plain($voc_obj->description) .'</span>';
        $is_desc = true;
      }

      if ($tcount || $show_empty_vocs) {
        $index = drupal_strtoupper(drupal_substr($voc_obj->name, 0, $count_group_vocabularies));
        $arr[$index][] = l($voc_obj->name, 'taxonomy_vtn/voc/'. $voc_obj->vid) . $count_terms . $voc_desc;
      }
    }

    // change locale to sort
    if (variable_get('taxonomy_vtn_use_setlocale', 1) == 1 && variable_get('taxonomy_vtn_server_support_setlocale', 0) == 1) {
      $my_locale = variable_get('taxonomy_vtn_locale_code', 'en_US.UTF8');
      setlocale(LC_ALL, $my_locale);
      ksort($arr, SORT_LOCALE_STRING);
      // set locale back to default
      setlocale(LC_ALL, NULL); // without this count($arr) not work properly
    }
    else {
      ksort($arr); //--- this not support locales
    }


    $attr = $show_index != 0 ? array() : array('class' => 'simple-list');
    $cnt = ceil(count($arr)/$count_column_vocs);
    $k = 0; $col = 0;
    foreach ($arr as $i => $t) {
      if ($k == 0 || ($k%$cnt) == 0 || $k == $cnt) {
        $cell = 'cell'. ++$col;
      }
      $$cell .= theme('item_list', $t, $show_index != 0 ? $i : NULL, 'ul', $attr);
      ++$k;
    }

    if ($k) {
      // prepare cells and their width
      ($col < $count_column_vocs ) ? $ile = $col : $ile = $count_column_vocs;
      for ($num=1; $num <= $ile; ++$num) {
        $xcell = 'cell'. $num;
        $xcells[] = array('data' => $$xcell, 'style' => 'width:'. (100/$ile) .'%');
      }
      $row[] = $xcells;


      if ($show_voc_desc && $is_desc) {
        $output .= '<div class="taxonomy-vtn-navigation">';
        $output .= '<a class="show-hide-desc" href="javascript:taxonomy_vtn_show_hide_desc(\'vocabularies\');">'. t('Show / Hide descriptions') .'</a>';
        $output .= '</div>';
      }

      $output .= theme('table', array(), $row, array('class' => 'taxonomy-vtn-vocabularies', 'id' => 'taxonomy_vtn_vocabularies'));

      // Cache the fully rendered page in case of aggressive caching
      if (variable_get('taxonomy_vtn_caching', 0) == 1) {
        cache_set('taxonomy_vtn_show_vocs', $output);
      }
    }
    else {
      $output .= '<p class="taxonomy-vtn-no-data-info">';
      $output .= t('No data to display. Probably you have set option to not show empty vocabularies.');
      $output .= '</p>';
    }
  }
  else {
    $output .= '<p class="taxonomy-vtn-no-data-info">';
    $output .= t('No data to display. Probably you not have any vocabularies');
    $output .= '</p>';
  }

  $output .= '</div>'; // close tag <div class="taxonomy-vtn-vocabulary-page">

  // Cache the fully rendered page in case of aggressive caching
  if (variable_get('taxonomy_vtn_caching', 0) == 1) {
    cache_set('taxonomy_vtn_show_vocs', $output);
  }

  return $output;
}

/**
 * Implementation of taxonomy_vtn_show_terms().
 */
function taxonomy_vtn_show_terms($arg = '') {

  // check for excluded voc_id
  $omit_vocs = _taxonomy_vtn_get_list('taxonomy_vtn_vocabularies_omit');
  if ($omit_vocs) {
    if (in_array($arg, $omit_vocs)) {
      $arg = NULL;
    }
  }

  if (!is_numeric($arg)) {
    drupal_set_message(t('Please select a vocabulary first'));
    drupal_goto('taxonomy_vtn');
  }

  $own_vid = (int)$arg;
  $voc_obj = taxonomy_vocabulary_load($own_vid);
  if (!$voc_obj) {
    drupal_set_title(t('Bad vocabulary id'));
    $output  = '<p class="taxonomy-vtn-no-data-info">';
    $output .= t('No data to display. You probably have no vocabulary with vid = !voc_id', array('!voc_id' => $own_vid));
    $output .= '</p>';
    $terms_navigation = '';
    if (variable_get('taxonomy_vtn_terms_show_back_to_voc', 1) == 1) {
      $where_back = t('vocabularies');
      if (variable_get('taxonomy_vtn_user_friendly_titles', 1) == 1) {
        $where_back = t('master index');
      }
      $terms_navigation = '<span class="taxonomy-vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn') .'</span>';
      $output .= '<div class="taxonomy-vtn-navigation">'. $terms_navigation .'</div>';
    }
    return $output;
  }


  $title = '';
  if (variable_get('taxonomy_vtn_user_friendly_titles', 1) == 1) {
    $title = t('Terms in') .' ';
  }
  drupal_set_title($title . t('@vocabulary', array('@vocabulary' => $voc_obj->name)));

  // Check if the page hasn't already been cached. If not, create and cache it.
  $cc = cache_get('taxonomy_vtn_show_terms_'. $own_vid);
  $cache_data = $cc->data;
  if ($cache_data && variable_get('taxonomy_vtn_caching', 0) == 1) {
    $output = $cache_data;
    //print 'z cache';
    return $output;
  }

  // vocabulary vid is ok, lets do it
  $output = '<div class="taxonomy-vtn-term-page">';
  $output .= '<div class="taxonomy-vtn-description">'. check_plain($voc_obj->description) .'</div>';

  $show_empty_terms = variable_get("taxonomy_vtn_terms_show_empty_terms$own_vid", 1);
  $show_index = variable_get("taxonomy_vtn_terms_show_terms_index$own_vid", 1);
  $show_count_nodes = variable_get("taxonomy_vtn_terms_show_count_nodes$own_vid", 1);
  $show_term_desc = variable_get("taxonomy_vtn_terms_show_term_desc$own_vid", 1);
  $show_synonyms = variable_get("taxonomy_vtn_terms_show_synonyms$own_vid", 1);
  $count_group_terms =  variable_get("taxonomy_vtn_terms_count_group_terms$own_vid", 1);
  $count_column_terms =  variable_get("taxonomy_vtn_terms_count_column_terms$own_vid", 2);
  $term_goto = variable_get("taxonomy_vtn_terms_goto$own_vid", 'nodes');
  $show_back_to_voc = variable_get("taxonomy_vtn_terms_show_back_to_voc$own_vid", 1);
  $show_related = variable_get("taxonomy_vtn_terms_show_related$own_vid", 0);
  $show_quick_index = variable_get("taxonomy_vtn_terms_show_quick_index$own_vid", 0);
  $break_quick_index = variable_get("taxonomy_vtn_terms_break_quick_index$own_vid", 0);

  // TODO: for depth use:  if ( $terms = taxonomy_get_tree($own_vid,0,-1,3) )
  if ( $terms = taxonomy_get_tree($own_vid) ) {
    $arr = array();
    $arr_terms = array();
    $row = array();
    $is_desc = false;

    // exclude some terms_id (use global and local term numbers)
    $omit_terms = _taxonomy_vtn_get_list('taxonomy_vtn_terms_omit',
                                         "taxonomy_vtn_terms_omit$own_vid");
    if (!$omit_terms) {
      // to make sure it works, use an empty array in this one case
      $omit_terms = array();
    }

    // only include some terms_id (use global and local term numbers)
    $include_terms = _taxonomy_vtn_get_list('taxonomy_vtn_terms_include',
                                            "taxonomy_vtn_terms_include$own_vid");

    // prepare own full structure with terms and synonyms and link to them
    require_once('taxonomy_vtn_overwrites.inc.php');
    foreach ($terms as $term) {

      // include/exclude user specified terms and avoid duplicates
      if (($include_terms === NULL || in_array($term->tid, $include_terms)) &&
          !in_array($term->tid, $omit_terms)) {
        $omit_terms[] = $term->tid;

        switch ($term_goto) {
        //case 'nodes':
        default:
          $term_goto_path = 'taxonomy_vtn/term/'. $term->tid;
          //$tcount = taxonomy_term_count_nodes($term->tid);
          $tcount = _taxonomy_vtn_count_nodes(array($term->tid), 'all');
          break;

        case 'taxonomy_term':
          $term_goto_path = taxonomy_term_path($term->tid); //'taxonomy/term/'. $term->tid;
          //$tcount = db_result(db_query("SELECT COUNT(nid) FROM {term_node} WHERE tid = %d", $term->tid));
          $tcount = _taxonomy_vtn_count_nodes(array($term->tid), 0);
          break;

        case 'taxonomy_term_all':
          $term_goto_path = taxonomy_term_path($term->tid) .'/all'; //'taxonomy/term/'. $term->tid .'/all';
          //$tcount = taxonomy_term_count_nodes($term->tid);
          $tcount = _taxonomy_vtn_count_nodes(array($term->tid), 'all');
          break;

        }

        // add the count when available and requested
        if ($show_count_nodes && $tcount > 0) {
          $count_nodes = ' ('. $tcount .')';
        }
        else {
          $count_nodes = '';
        }

        $term_desc = '';
        if ($show_term_desc && $term->description) {
          $term_desc = '<br /><span class="description">'. check_plain($term->description) .'</span>';
          $is_desc = true;
        }

        if ($tcount || $show_empty_terms) {
          $arr_terms[$term->name]['term'][] = l($term->name, $term_goto_path) . $count_nodes . $term_desc ;
          // if is not empty term or we show empty terms
          // and we will show synonyms then get for this
          if ($show_synonyms) {
            $synonyms = taxonomy_get_synonyms($term->tid);
            foreach ($synonyms as $synonym_name) {
              // $synonym_term = taxonomy_get_synonym_root($synonym_name);
              // all synonyms have a target - this term
              $arr_terms[$synonym_name]['synonym'][] = l($term->name, $term_goto_path);
            }
          }

          if ($show_related) {
            // prepare related terms
            $related = taxonomy_get_related($term->tid);
            foreach ($related as $rel) {

              switch ($term_goto) {
              case 'nodes':
                $rel_goto_path = 'taxonomy_vtn/term/'. $rel->tid;
                break;

              case 'taxonomy_term':
                $rel_goto_path = taxonomy_term_path($rel->tid); //'taxonomy/term/'. $rel->tid;
                break;

              case 'taxonomy_term_all':
                $rel_goto_path = taxonomy_term_path($rel->tid) .'/all'; //'taxonomy/term/'. $rel->tid .'/all';
                break;

              }

              $relatedx[$term->name][] = l($rel->name, $rel_goto_path);
            }
          } // show realted
        } // has terms or show empty true
      } // exclude end
    } // end foreach terms as term prepare


    $my_locale = variable_get('taxonomy_vtn_locale_code', 'en_US.UTF8');

    // change locale to sort
    if (variable_get('taxonomy_vtn_use_setlocale', 1) == 1 && variable_get('taxonomy_vtn_server_support_setlocale', 0) == 1 ) {
      setlocale(LC_ALL, $my_locale);
      ksort($arr_terms, SORT_LOCALE_STRING);
      // set locale back to default
      setlocale(LC_ALL, NULL); // without this count($arr) not work properly
    }
    else {
      ksort($arr_terms); //--- this not support languages
    }

    // Now we make index with links to proper terms for synonyms
    foreach ($arr_terms as $name => $types) {
      $index = drupal_strtoupper(drupal_substr($name, 0, $count_group_terms));
      foreach ($types as $type => $links) {
        if ($type == 'term') {

          // if some related terms exist then we show them
          $ree = '';
          if ($show_related && $relatedx) {
            $related_links = '';
            foreach ($relatedx as $name2 => $a) {
              if ($name2 == $name ) {
                foreach ($a as $f){
                  $related_links[] = $f;
                }
              }
            }

            if( !empty($related_links) ) {
              $ree = '<br /> <small>'. t('Related terms') .': '. implode(', ', $related_links) .'</small>';
            }
          }

          foreach ($links as $link) { // maybe is more than one
            $arr[$index][] = $link . $ree;
          }
        }
        else { // is a synonym then look for terms
          $x_links = array();
          foreach ($links as $link) { // propably is more than one
            $x_links[] = $link;
          }
          $arr[$index][] = $name .'<br /> <small>'. t('Synonyms') .': '. implode(', ', $x_links) .'</small>';
        }
      }
    } // end foreach arr_terms


    // sort array key (index like A B C .. etc.)
    // change locale to sort
    if (variable_get('taxonomy_vtn_use_setlocale', 1) == 1 && variable_get('taxonomy_vtn_server_support_setlocale', 0) == 1) {
      setlocale(LC_ALL, $my_locale);
      ksort($arr, SORT_LOCALE_STRING);
      // set locale back to default
      setlocale(LC_ALL, NULL); // without this count($arr) not work properly
    }
    else {
      ksort($arr); //--- not support for languages
    }

    // for the top letters index
    $href = null;
    if ($break_quick_index && $show_quick_index) {
      $cut_href = ceil(count($arr) / 2);
    }
    else {
      $cut_href = 0;
    }
    $href_name = '';
    $href_close = '';

    $cnt = ceil(count($arr) / $count_column_terms);
    $k = 0;
    $col = 0;
    foreach ($arr as $i => $t) {
      if ($k == 0 || ($k%$cnt) == 0 || $k == $cnt) {
        $cell = 'cell'. ++$col;
      }
      if($show_index == 0) {
        $$cell .= theme('item_list', $t, NULL, 'ul', array('class' => 'simple-list'));
      }
      else {
        if ($show_quick_index) {
          if ($cut_href > 0) {
            --$cut_href;
            if ($cut_href == 0) {
              $href .= '<br/>';
            }
          }
          // TODO: we would need to enhance the test and make sure that $i is a valid character for HREF
          switch ($i) {
          case '&': $href_letter = 'amp'; break;
          case '"': $href_letter = 'quot'; break;
          case '\'': $href_letter = 'apos'; break;
          default: $href_letter = $i; break;
          }
          $href .= '<a class="taxonomy-vtn-index-link" href="#name-taxonomy-vtn-term-' . $href_letter . '">' . $i . '</a> ';
          $href_name = '<a class="taxonomy-vtn-index-anchor" name="name-taxonomy-vtn-term-' . $href_letter . '">';
          $href_close = '</a>';
        }
        $$cell .= theme('item_list', $t, $href_name . $i . $href_close, 'ul');
      }
      ++$k;
    }

    if ($k) {
      // prepare cells and their width
      ($col < $count_column_terms ) ? $ile = $col : $ile = $count_column_terms;
      for ($num=1; $num <= $ile; ++$num) {
        $xcell = 'cell'. $num;
        $xcells[] = array('data' => $$xcell, 'style' => 'width:'. (100/$ile) .'%');
      }
      $row[] = $xcells;

      $terms_navigation = array();
      if ($show_back_to_voc) {
        $where_back = t('vocabularies');
        if (variable_get('taxonomy_vtn_user_friendly_titles', 1) == 1) {
          $where_back = t('master index');
        }
        $terms_navigation[] = '<span class="taxonomy-vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn') .'</span>';
      }

      if ($show_term_desc && $is_desc) {
        $terms_navigation[] = '<a class="show-hide-desc" href="javascript:taxonomy_vtn_show_hide_desc(\'terms\');">'. t('Show / Hide descriptions') .'</a>';
      }

      if ($show_back_to_voc || $show_term_desc) {
        $output .= '<div class="taxonomy-vtn-navigation">'. implode(' &nbsp; | &nbsp; ', $terms_navigation) .'</div>';
      }
      if ($href) {
        $output .= '<div class="taxonomy-vtn-references">' . $href . '</div>';
      }

      $output .= theme('table', array(), $row, array('class' => 'taxonomy-vtn-terms', 'id' => 'taxonomy_vtn_terms'));
    }
    else {
      $output .= '<p class="taxonomy-vtn-no-data-info">';
      $output .= t('No data to display. You probably set the option to avoid showing empty terms.');
      $output .= '</p>';
      $terms_navigation = '';
      if ($show_back_to_voc) {
        $where_back = t('vocabularies');
        if (variable_get('taxonomy_vtn_user_friendly_titles', 1) == 1) {
          $where_back = t('master index');
        }
        $terms_navigation = '<span class="taxonomy-vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn') .'</span>';
        $output .= '<div class="taxonomy-vtn-navigation">'. $terms_navigation .'</div>';
      }
    }
  }// end if terms
  else {
    $output .= '<p class="taxonomy-vtn-no-data-info">';
    $output .= t('No data to display. This vocabulary is probably empty.');
    $output .= '</p>';
    $terms_navigation = '';
    if ($show_back_to_voc) {
      $where_back = t('vocabularies');
      if (variable_get('taxonomy_vtn_user_friendly_titles', 1) == 1) {
        $where_back = t('master index');
      }
      $terms_navigation = '<span class="taxonomy-vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn') .'</span>';
      $output .= '<div class="taxonomy-vtn-navigation">'. $terms_navigation .'</div>';
    }
  }

  $output .= '</div>'; // closing tag <div class="taxonomy-vtn-term-page">

  // Cache the fully rendered page in case of aggressive caching
  if (variable_get('taxonomy_vtn_caching', 0) == 1) {
    cache_set('taxonomy_vtn_show_terms_'. $own_vid, $output);
  }

  return $output;
}


/**
 * Implementation of taxonomy_vtn_show_nodes().
 */
function taxonomy_vtn_show_nodes($arg = '') {

  // check for globally excluded voc_id
  $omit_terms = _taxonomy_vtn_get_list('taxonomy_vtn_terms_omit');
  if ($omit_terms) {
    if (in_array($arg, $omit_terms)) {
      $arg = NULL;
    }
  }

  if (!is_numeric($arg)) {
    drupal_set_message(t('Please select a vocabulary first'));
    drupal_goto('taxonomy_vtn');
  }

  $own_tid = (int)$arg;
  $term_obj = taxonomy_get_term($own_tid);
  if (!$term_obj) {
    drupal_set_title(t('Bad term id'));
    $output  = '<p class="taxonomy-vtn-no-data-info">';
    $output .= t('No data to display. Probably you not have a term with tid = !term_id', array('!term_id' => $own_tid));
    $output .= '</p>';

    $terms_navigation = '';
    if (variable_get('taxonomy_vtn_terms_show_back_to_voc', 1) == 1) {
      $where_back = t('vocabularies');
      if (variable_get('taxonomy_vtn_user_friendly_titles', 1) == 1) {
        $where_back = t('master index');
      }
      $terms_navigation = '<span class="taxonomy-vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn') .'</span>';
      $output .= '<div class="taxonomy-vtn-navigation">'. $terms_navigation .'</div>';
    }
    return $output;
  }

  $title = '';
  if (variable_get('taxonomy_vtn_user_friendly_titles', 1) == 1) {
    $title = t('Nodes of term') .' ';
  }
  drupal_set_title($title . t('@term_name', array('@term_name' => $term_obj->name)));

  // Check if the page hasn't already been cached. If not, create and cache it.
  $cc = cache_get('taxonomy_vtn_show_nodes_'. $own_tid);
  $cache_data = $cc->data;
  if ($cache_data && variable_get('taxonomy_vtn_caching', 0) == 1) {
    $output = $cache_data;
    //print 'z cache';
    // Got a problem with the cache and the pager. We'd need to know whether it is necessary
    // to show the page here.
    //$output .= theme('pager');  // this is not saved in the cache since it is always dynamic
    return $output;
  }

  // the variables are saved on a per vocabulary basis, get the identifier
  $own_vid = $term_obj->vid;
  $show_count_comments = variable_get("taxonomy_vtn_nodes_show_count_comments$own_vid", 1);
  $count_column_nodes = variable_get("taxonomy_vtn_nodes_count_column_nodes$own_vid", 2);
  $show_empty_count_comments = variable_get("taxonomy_vtn_nodes_show_empty_count_comments$own_vid", 0);
  $show_index = variable_get("taxonomy_vtn_nodes_show_nodes_index$own_vid", 0);
  $show_back_to_terms = variable_get("taxonomy_vtn_nodes_show_back_to_terms$own_vid", 1);

  if ($use_pager = (bool)variable_get("taxonomy_vtn_nodes_use_pager$own_vid", FALSE) === TRUE) {
    $links_per_page = variable_get("taxonomy_vtn_nodes_links_per_page$own_vid", 10);
  }
  else {
    $links_per_page = 0;
  }

  // TODO: settings for depth , probably use level of terms ?
  require_once('taxonomy_vtn_overwrites.inc.php');
  $nodes = _taxonomy_vtn_select_nodes(array($own_tid), $operator = 'or', $depth = 'all', $use_pager, $order = 'n.title, n.sticky DESC', $links_per_page);
  while ($node = db_fetch_object($nodes)) {
    $node = node_load($node->nid);
    $title = ltrim($node->title);
    if (strlen($title) == 0) {
      $index = '&mdash;';
    }
    else {
      $first_letter = drupal_substr($title, 0, 1);
      if ($first_letter == '&') {
        // HTML entity, keep as is
        $index = preg_replace('/;.*/', ';', $title);
      }
      else {
        // TODO: cut "special" chars (does it really cut UTF-8 badly? TBD)
        $index = drupal_strtoupper($first_letter);
      }
    }
    //print_r($node);
    if ($show_count_comments) {
      if ( $node->comment_count || $show_empty_count_comments) {
        $count_comments = ' ('. format_plural($node->comment_count, '1 comment', '@count comments') .')';
      }
      else {
        $count_comments = '';
      }
    }
    $arr[$index][] = l($node->title, 'node/'. $node->nid) . $count_comments;
    // for <li> style .... $arr[$index][] = array('data' => l($node->title,'node/'.$node->nid) . $count_comments, 'style' => '');
  }

  $output = '<div class="taxonomy-vtn-node-page">';
  if ( count($arr) > 0 ) {

    $my_locale = variable_get('taxonomy_vtn_locale_code', 'en_US.UTF8');

    // change locale to sort
    if (variable_get('taxonomy_vtn_use_setlocale', 1) == 1 && variable_get('taxonomy_vtn_server_support_setlocale', 0) == 1) {
      setlocale(LC_ALL, $my_locale);
      ksort($arr, SORT_LOCALE_STRING);
      // set locale back to default
      setlocale(LC_ALL, NULL); // without this count($arr) not work properly
    }
    else {
      ksort($arr); //--- this not support locales
    }

    $attr = $show_index != 0 ? array() : array('class' => 'simple-list');
    $cnt = ceil(count($arr)/$count_column_nodes);
    $k = 0; $col = 0;
    foreach ($arr as $i => $t) {
      if ($k == 0 || ($k%$cnt) == 0 || $k == $cnt) {
        $cell = 'cell'. ++$col;
      }
      $$cell .= theme('item_list', $t, $show_index != 0 ? $i : NULL, 'ul', $attr);
      ++$k;
    }

    // prepare cells and their width
    $ile = $col < $count_column_nodes ? $col : $count_column_nodes;
    for ($num=1; $num <= $ile; ++$num) {
      $xcell = 'cell'. $num;
      $xcells[] = array('data' => $$xcell, 'style' => 'width:'. (100/$ile) .'%');
    }
    $row[] = $xcells;

    $output .= '<div class="description">'. $term_obj->description .'</div>';

    $nodes_navigation = '';
    if ($show_back_to_terms) {
      $where_back = t('terms');
      if (variable_get('taxonomy_vtn_user_friendly_titles', 1) == 1) {
        $where_back = t('master index');
      }
      $nodes_navigation = '<span class="taxonomy-vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn/voc/'. $term_obj->vid) .'</span>';
      $output .= '<div class="taxonomy-vtn-navigation">'. $nodes_navigation .'</div>';
    }

    $output .= theme('table', array(), $row, array('class' => 'taxonomy-vtn-nodes', 'id' => 'taxonomy_vtn_nodes'));
  }
  else {
    $output .= '<p class="taxonomy-vtn-no-data-info">';
    $output .= t('No data to display. This term is probably not used by any node.');
    $output .= '</p>';
    $nodes_navigation = '';
    if ($show_back_to_terms) {
      $where_back = t('terms');
      if (variable_get('taxonomy_vtn_user_friendly_titles', 1) == 1) {
        $where_back = t('master index');
      }
      $nodes_navigation = '<span class="taxonomy-vtn-back-button">'. l('<< '. t('Back to') .' '. $where_back, 'taxonomy_vtn/voc/'. $term_obj->vid) .'</span>';
      $output .= '<div class="taxonomy-vtn-navigation">'. $nodes_navigation .'</div>';
    }
  }

  $output .= '</div>';  // close tag <div class="taxonomy-vtn-node-page">

  // Cache the fully rendered page in case of aggressive caching
  if (variable_get('taxonomy_vtn_caching', 0) == 1) {
    cache_set('taxonomy_vtn_show_nodes_'. $own_tid, $output);
  }

  // We would need to save that count since on the next call with a
  // cache we will need to add the pager again.
  if ( count($arr) > 0 ) {
    $output .= theme('pager');
  }

  return $output;
}

